package example;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import processing.StatusListener;

public class Main {

	public static void main(String[] args) {
		// Niech zmienna p reprezentuje referencj do jakiej nieznanej klasy, 
	    // implementujcej interfejs Processor, zaadowanej wasnym adowaczem.
		// Jednak w tym przykadowym kodzie nie dostarczono adnego takiego adowacza klas.
		// Zamiast tego do p przypisano warto wyuskana bezporednio z klasy MyProcessor
		// (implementujcej interfejs Processor, ale znanej na etapie kompilacji projektu).
		// W ten sposb "zasymulowano" adowanie klasy.
		Class<?> p = MyProcessor.class;
		// Majc zaadowan klas mona sign do jej metod i je wywoa. 
		try {
			// Pozyskujemy konstruktor
			Constructor<?> cp = p.getConstructor();
			// a nastpnie tworzymy obiekt zaadowanej klasy 
			// (trzeba to zrobi, aby mona byo wywoa metody instancyjne).
			Object o = cp.newInstance();
			
			
			
			// Poniewa wiemy, e zaadowana klasa implementuje interfejs processing.Processor, 
			// dlatego szukamy w tej klasie znanych metod.
			// Najpierw dowiadujemy si, na czym polega algorytm przetwarzania wywoujc getInfo()
			Method getInfoMethod = p.getDeclaredMethod("getInfo");
			System.out.println((String)getInfoMethod.invoke(o));

			// Nastpnie przesyamy zadanie do wykonania metod submitTask()
			// Method method = p.getDeclaredMethod("submitTask", new Class[] {String.class,
			// StatusListener.class});					
			Method submitTaskMethod = p.getDeclaredMethod("submitTask", String.class, StatusListener.class);
			
			// Aby wywoa t metod naley posuy si odpowiednimi parametrami: task, sl
			// gdzie sl referencja do wasnego suchacza - instancji klasy implementujcej interfejs StatusListener.			
              
			// boolean b = (boolean) method.invoke(o,new Object[] {"Text to process", new
			// MyStatusListener()});
			boolean b = (boolean) submitTaskMethod.invoke(o, "Tekst na wejcie", new MyStatusListener());

			// Implementacja metody submitTask w klasie MyProcessor jest asynchroniczna.
			// Do rozpoznania koca przetwarzania i pobrania wyliczonego wyniku 
			// naley wykorzysta metody suchacza.
			// Na razie piszemy tylko komunikat, e zainicjowano przetwarzanie
			if (b)
				System.out.println("Processing started correctly");
			else
				System.out.println("Processing ended with an error");

			// Do rozpoznania koca przetwarzania suy metoda statusChanged
			// suchacza MyStatusListener. Do tej metody dostarczana bdzie instancja Status,
			// z ktrej wycign mona progress i taskId.
			// Jak progress dojdzie do koca, bdzie mona pobra wynik przetwarzania
			// getod getResult() z instancji klasy MyProcessor.
			// Jeli z metody getResult() otrzyma si co innego ni null, to wanie bdzie to
            // wynik

			// Poniewa mamy program konsolowy, asynchroniczno wymaga uruchamieniam wtku
			// oczekujcego na rezultat (sprawdzajcy, czy przetwarzanie nie dobiego koca).
			// W aplikacji okienkowej podobnie zadziaaby suchacz, ktry 
			// odpaliby pobranie rezultatu.

			
			Method getResultMethod = p.getDeclaredMethod("getResult");

			ExecutorService executor = Executors.newSingleThreadExecutor();
			
			// uruchom zadanie, ktre skoczy si, gdy result!=null
			executor.submit(() -> {
				String result = null;				
				while (true) {
					// System.out.println(scheduleFuture.isDone());
					
						try {
							Thread.sleep(800);

						// String result = (String) getResultMethod.invoke(o,new Object[] {});
							result = (String) getResultMethod.invoke(o);
						} catch (InterruptedException | IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
							e.printStackTrace();
						}
						if (result != null) {
							System.out.println("Result: " + result);
							executor.shutdown();
							break;
						}
				}
			});

			System.out.println("main FINISHED");

		} catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
				| IllegalArgumentException | InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
