package example;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;


import processing.Processor;
import processing.Status;
import processing.StatusListener;

/**
 * Klasa procesora symulujca dugotrwae przetwarzanie
 * @author tkubik
 *
 */
		
public class MyProcessor implements Processor {
	
	private static int taskId=0;
	private String result = null;

	@Override
	public boolean submitTask(String task, StatusListener sl) {
		taskId++;
		AtomicInteger ai = new AtomicInteger(0);
		 	
		ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();
		// Aby zasymulowa dugotrwae przetwarzanie uruchamiane s w nieskoczono
		// krtkie zadania inkrementujce licznik i nie zwracajce wartoci.
		// Waciwie powinno si przetwarza przekazywany task, ale tutaj nie jest to robione.
		//
		// ScheduledFuture<?> scheduleFuture = executorService.scheduleAtFixedRate(

		executorService.scheduleAtFixedRate(
				()->{
					System.out.println("running"); // do debbugowania
		            ai.incrementAndGet();
		            sl.statusChanged(new Status(taskId,ai.get()));		    		
				}, 
				1, 10, TimeUnit.MILLISECONDS);	
		
		// Powysze mona byoby zrobi w ptli 
		//  for(int i=1; i<=100; i++){
		//     try {
		//         ai.incrementAndGet();
		//        Thread.sleep(1000);
		//     } catch(InterruptedException e){System.out.println(e);}
        //     sl.statusChanged(new Status(taskId,ai.get()));
		// ale zrobiono to inaczej. 
		
		// Poniewa zapuszczono egzekutor, trzeba poczeka, a przekrci 
		// si w nim 100 zada, po czym naley zamkn serwis egzekutora 
		// i zakoczy dziaanie samego egzekutora.
		// Mona to zrobi "na zewntrz", w kolejnym egzekutorze.
		// Przykad ten nie jest moe najlepszy, ale chodzio w nim 
		// o pokazanie synchronizacji przez zmienn wspdzielon.

		ExecutorService executor = Executors.newSingleThreadExecutor();
		// uruchom zadanie, ktre skoczy si, gdy licznik przekroczy warto 100
		executor.submit(() -> {
		      while (true) {
		          //System.out.println(scheduleFuture.isDone()); will always print false
		          try {
					Thread.sleep(10);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		          if (ai.get() >= 100) { 
		        	  // przekrcilimy 100 razy, 
		        	  // moemy zwrci wynik przetwarzania
		        	 // tutaj "przetwarzanie" polega na zamianie literek na due
		        	  result = task.toUpperCase(); 
		              System.out.println("finished");
		              //scheduleFuture.cancel(true);
		              executorService.shutdown();
		              executor.shutdown();
		              break;
		          }
		      }			
			});
		
		// Jeli chcielibymy zaznaczy, e co poszo nie tak 
		// naleaoby zwrci false
		// (np. gdy ostatnie przetwarzanie jeszcze nie dobiego koca)
		// 
		// Jeli wszystko poszo ok, wtedy zwracane jest true
	    //
		// Uwaga: przypominajc - implementacja "przetwarzania" jest asynchroniczna,
		//        zwrcenie wartoci true naley interpretowa jako informacj o tym,
		//        e zlecenie przetwarzania si udao.
		return true;
	}

	@Override
	public String getInfo() {
		return "Zamiana literek na due";
	}

	@Override
	public String getResult() {
		return result;
	}

}
